@@ -14,7 +14,7 @@ BEGIN {
 
 use Graph::AdjacencyMap qw(:flags :fields);
 
-our $VERSION = '0.9726';
+our $VERSION = '0.9728';
 
 require 5.006; # Weak references are absolutely required.
 
@@ -1825,6 +1825,29 @@ sub directed_copy {
 
 *directed_copy_graph = \&directed_copy;
 
+sub is_bipartite {
+    &expect_undirected;
+    my $g = $_[0];
+
+    my $is_bipartite = 1;
+    my %colors;
+    my $operations = {
+        tree_edge => sub {
+            my( $seen, $unseen ) = @_;
+            ( $seen, $unseen ) = sort { exists $colors{$b} <=> exists $colors{$a} } ( $seen, $unseen );
+            $colors{$seen} ||= -1;
+            $colors{$unseen} = -$colors{$seen};
+        },
+        non_tree_edge => sub {
+            $is_bipartite = '' if $colors{$_[0]} == $colors{$_[1]};
+        },
+    };
+
+    require Graph::Traversal::DFS;
+    Graph::Traversal::DFS->new( $g, %$operations )->dfs;
+    return $is_bipartite;
+}
+
 ###
 # Cache or not.
 #
@@ -2114,14 +2137,13 @@ sub _biconnectivity_out {
 }
 
 sub _biconnectivity_dfs {
-  my ($g, $u, $state) = @_;
+  my ($E, $u, $state) = @_;
   $state->{low}{$u} = $state->{num}{$u} = $state->{dfs}++;
-  for my $v ($g->successors($u)) {
+  for my $v ($E->successors($u)) {
     if (!exists $state->{num}{$v}) {
       push @{$state->{stack}}, [$u, $v];
       $state->{pred}{$v} = $u;
-      $state->{succ}{$u}{$v}++;
-      _biconnectivity_dfs($g, $v, $state);
+      _biconnectivity_dfs($E, $v, $state);
       $state->{low}{$u} = List::Util::min(@{ $state->{low} }{$u, $v});
       _biconnectivity_out($state, $u, $v)
 	if $state->{low}{$v} >= $state->{num}{$u};
@@ -2137,11 +2159,12 @@ sub _biconnectivity_dfs {
 sub _biconnectivity_compute {
     require List::Util;
     my ($g) = @_;
+    my ($V, $E) = @$g[ _V, _E ];
     my %state = (BC=>[], dfs=>0);
-    my @u = $g->vertices;
+    my @u = $V->ids;
     for my $u (@u) {
 	next if exists $state{num}->{$u};
-	_biconnectivity_dfs($g, $u, \%state);
+	_biconnectivity_dfs($E, $u, \%state);
 	push @{$state{BC}}, delete $state{stack} if @{ $state{stack} || _empty_array };
     }
 
@@ -2172,7 +2195,8 @@ sub _biconnectivity_compute {
 
     # Create the subgraph components.
     my @sg = map [ List::Util::uniq( map @$_, @$_ ) ], @{$state{BC}};
-    return [ \@ap, \@sg, \@br, \%v2bc, \%v2bc_vec, $Z ];
+    my ($apdeep, $sgv, $brv) = $V->get_paths_by_ids([[\@ap], \@sg, \@br], 1);
+    return [ @$apdeep, $sgv, $brv, \%v2bc, \%v2bc_vec, $Z ];
 }
 
 sub biconnectivity {
@@ -2212,12 +2236,16 @@ sub biconnected_component_by_vertex {
     my ($v) = splice @_, 1, 1;
     my $v2bc = (&biconnectivity)[3];
     splice @_, 1, 0, $v;
+    my $V = $_[0]->[ _V ];
+    ($v) = $V->get_ids_by_paths([$v]);
     return defined $v2bc->{ $v } ? keys %{ $v2bc->{ $v } } : ();
 }
 
 sub same_biconnected_components {
     my ($v2bc, $Z) =  (&biconnectivity)[4,5];
-    return 0 if grep !defined, my @vecs = @$v2bc{ @_[1..$#_] };
+    my $V = $_[0]->[ _V ];
+    my @vs = $V->get_ids_by_paths([@_[1..$#_]]);
+    return 0 if grep !defined, my @vecs = @$v2bc{ @vs };
     my $accumulator = $vecs[0];
     $accumulator &= $_ for @vecs[1..$#vecs]; # accumulate 0s -> all in same
     $accumulator ne $Z;
